import React from 'react';
import { EditorView } from '@codemirror/view';
import { UseCodeMirrorProps } from './hooks/useCodeMirror';
import { ToolbarInstance, ToolbarProps } from './Toolbar';
import { PreviewInstance, PreviewProps as PreviewOptions } from './Preview';
import './Editor.less';
export interface EditorInstance {
    getContainer(): HTMLDivElement | null;
    getEditorView(): EditorView | null;
    getToolbar(): ToolbarInstance | null;
    getPreview(): PreviewInstance | null;
    getDisplayMode(): {
        fullScreen: boolean;
        preview: boolean;
        editorPreview: boolean;
    };
    toggleFullScreen(bool?: boolean): void;
    togglePreview(bool?: boolean): void;
    toggleEditorPreview(bool?: boolean): void;
}
export interface EditorProps extends Omit<UseCodeMirrorProps, 'containerRef' | 'onUpdate'> {
    /**
     * @description  工具条
     */
    toolbar?: false | {
        buttons: ToolbarProps['buttons'];
    };
    /**
     * @description 编辑预览启用全屏，一般和 `height` 搭配使用
     * @default true
     */
    editorPreviewEnableFullScreen?: boolean;
    /**
     * @description 预览选项
     */
    previewOptions?: PreviewOptions;
    /**
     * @description 自动保存，下次初始化时可恢复上一次保存的内容
     */
    storageKey?: string;
}
declare const Editor: React.ForwardRefExoticComponent<EditorProps & React.RefAttributes<EditorInstance>>;
export default Editor;
