var _excluded = ["toolbar", "editorPreviewEnableFullScreen", "previewOptions", "storageKey"];

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import React, { forwardRef, useEffect, useImperativeHandle, useRef } from 'react';
import { keymap } from '@codemirror/view';
import classnames from 'classnames';
import useCodeMirror from './hooks/useCodeMirror';
import useDisplayMode from './hooks/useDisplayMode';
import useBackup from './hooks/useBackup';
import * as commands from './commands';
import Toolbar from './Toolbar';
import Preview from './Preview';
import "./Editor.css";
var Editor = /*#__PURE__*/forwardRef(function (props, ref) {
  var containerRef = useRef(null);
  var toolbarRef = useRef(null);
  var previewRef = useRef(null);
  var height = props.height,
      minHeight = props.minHeight,
      maxHeight = props.maxHeight;

  var toolbar = props.toolbar,
      editorPreviewEnableFullScreen = props.editorPreviewEnableFullScreen,
      previewOptions = props.previewOptions,
      storageKey = props.storageKey,
      useCodeMirrorProps = _objectWithoutProperties(props, _excluded);

  var editorViewRef = useCodeMirror(_objectSpread(_objectSpread({}, useCodeMirrorProps), {}, {
    containerRef: containerRef,
    onUpdate: function onUpdate(viewUpdate) {
      if (viewUpdate.docChanged) {
        if (displayModeRef.current.editorPreview) {
          var _previewRef$current;

          var value = viewUpdate.state.doc.toString();
          (_previewRef$current = previewRef.current) === null || _previewRef$current === void 0 ? void 0 : _previewRef$current.debounceSetMarkdown(value);
        }
      }
    }
  }));

  var _useDisplayMode = useDisplayMode({}, editorPreviewEnableFullScreen),
      _useDisplayMode2 = _slicedToArray(_useDisplayMode, 4),
      displayModeRef = _useDisplayMode2[0],
      toggleFullScreen = _useDisplayMode2[1],
      togglePreview = _useDisplayMode2[2],
      toggleEditorPreview = _useDisplayMode2[3];

  var _displayModeRef$curre = displayModeRef.current,
      fullScreen = _displayModeRef$curre.fullScreen,
      preview = _displayModeRef$curre.preview,
      editorPreview = _displayModeRef$curre.editorPreview;
  useBackup(editorViewRef, storageKey);
  useImperativeHandle(ref, function () {
    return {
      getContainer: function getContainer() {
        return containerRef.current;
      },
      getEditorView: function getEditorView() {
        return editorViewRef.current;
      },
      getToolbar: function getToolbar() {
        return toolbarRef.current;
      },
      getPreview: function getPreview() {
        return previewRef.current;
      },
      getDisplayMode: function getDisplayMode() {
        return displayModeRef.current;
      },
      toggleFullScreen: toggleFullScreen,
      togglePreview: togglePreview,
      toggleEditorPreview: toggleEditorPreview
    };
  });
  useEffect(function () {
    if (editorViewRef.current) {
      var _toolbarRef$current;

      (_toolbarRef$current = toolbarRef.current) === null || _toolbarRef$current === void 0 ? void 0 : _toolbarRef$current.setKeymap(editorViewRef.current.state.facet(keymap).flat());
    }
  }, [editorViewRef.current]); // 预览

  useEffect(function () {
    if (preview || editorPreview) {
      var _editorViewRef$curren, _previewRef$current2;

      var value = (_editorViewRef$curren = editorViewRef.current) === null || _editorViewRef$curren === void 0 ? void 0 : _editorViewRef$curren.state.doc.toString();
      (_previewRef$current2 = previewRef.current) === null || _previewRef$current2 === void 0 ? void 0 : _previewRef$current2.setMarkdown(value || '');
    }
  }, [preview, editorPreview]); // 预览、全屏、编辑预览切换

  useEffect(function () {
    var _toolbarRef$current2;

    (_toolbarRef$current2 = toolbarRef.current) === null || _toolbarRef$current2 === void 0 ? void 0 : _toolbarRef$current2.setButtonState(displayModeRef.current);
  }, [displayModeRef.current]);

  var toolbarProps = _objectSpread({
    buttons: ['bold', 'italic', 'strikethrough', 'heading', '|', 'quote', 'code', 'table', 'line', 'unorderedList', 'orderedList', '|', 'link', 'image', '|', 'preview', 'editorPreview', 'fullScreen']
  }, toolbar);

  var handleToolbarButtonClick = function handleToolbarButtonClick(type) {
    var _commands$type, _editorViewRef$curren2;

    switch (type) {
      case 'fullScreen':
        return toggleFullScreen();

      case 'preview':
        return togglePreview();

      case 'editorPreview':
        return toggleEditorPreview();

      default:
        if (preview) return;
        (_commands$type = commands[type]) === null || _commands$type === void 0 ? void 0 : _commands$type.call(commands, editorViewRef.current);
        return (_editorViewRef$curren2 = editorViewRef.current) === null || _editorViewRef$curren2 === void 0 ? void 0 : _editorViewRef$curren2.focus();
    }
  };

  return /*#__PURE__*/React.createElement("div", {
    className: classnames('pf-mde-container', {
      'pf-mde-full-screen': fullScreen,
      'pf-mde-full-screen-mode': fullScreen,
      'pf-mde-editor-preview': editorPreview,
      'pf-mde-editor-preview-mode': editorPreview,
      'pf-mde-preview': preview,
      'pf-mde-preview-mode': preview
    })
  }, toolbar !== false && /*#__PURE__*/React.createElement(Toolbar, _objectSpread(_objectSpread({
    ref: toolbarRef
  }, toolbarProps), {}, {
    onButtonClick: handleToolbarButtonClick
  })), /*#__PURE__*/React.createElement("div", {
    className: "pf-mde-body",
    style: {
      height: height,
      minHeight: minHeight,
      maxHeight: maxHeight
    }
  }, /*#__PURE__*/React.createElement("div", {
    ref: containerRef,
    className: classnames('pf-mde-editor-container', {
      'pf-mde-hidden': preview
    })
  }), /*#__PURE__*/React.createElement("div", {
    className: classnames('pf-mde-preview-container', {
      'pf-mde-hidden': !preview && !editorPreview
    })
  }, /*#__PURE__*/React.createElement(Preview, _objectSpread({
    ref: previewRef,
    shouldUpdate: preview || editorPreview
  }, previewOptions)))));
});
export default Editor;