function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { useEffect, useMemo, useRef } from 'react';
import { EditorState, StateEffect } from '@codemirror/state';
import { EditorView, keymap } from '@codemirror/view';
import { markdown, markdownLanguage } from '@codemirror/lang-markdown';
import { languages } from '@codemirror/language-data';
import * as commands from '../commands';

function strSuffix(str) {
  var suffix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'px';
  return str ? String(str).replace(new RegExp("(".concat(suffix, ")?s*$"), 'i'), suffix) : '';
}

export default (function (props) {
  var containerRef = props.containerRef,
      value = props.value,
      _props$height = props.height,
      height = _props$height === void 0 ? '' : _props$height,
      _props$minHeight = props.minHeight,
      minHeight = _props$minHeight === void 0 ? '' : _props$minHeight,
      _props$maxHeight = props.maxHeight,
      maxHeight = _props$maxHeight === void 0 ? '' : _props$maxHeight,
      onChange = props.onChange,
      onUpdate = props.onUpdate;
  var editorViewRef = useRef(null);
  var onChangeRef = useRef();
  var onUpdateRef = useRef();
  onChangeRef.current = onChange;
  onUpdateRef.current = onUpdate;
  var extensions = useMemo(function () {
    return (props.extensions || []).concat([keymap.of([{
      key: 'Tab',
      run: commands.indent,
      shift: commands.indentLess
    }, {
      key: 'Mod-b',
      run: commands.bold
    }, {
      key: 'Mod-i',
      run: commands.italic
    }, {
      key: 'Mod-h',
      run: commands.heading
    }, {
      key: 'Mod-Alt-i',
      run: commands.image
    }, {
      key: 'Mod-\'',
      run: commands.quote
    }, {
      key: 'Mod-Alt-c',
      run: commands.code
    }, {
      key: 'Mod-l',
      run: commands.unorderedList
    }, {
      key: 'Mod-Alt-l',
      run: commands.orderedList
    }]), markdown({
      codeLanguages: languages,
      base: markdownLanguage
    }), EditorView.theme(_objectSpread(_objectSpread({
      '&': {
        height: strSuffix(height)
      }
    }, height ? {
      '.cm-scroller': {
        overflow: 'auto'
      }
    } : {}), {}, {
      '.cm-content, .cm-gutter': {
        minHeight: strSuffix(minHeight),
        maxHeight: strSuffix(maxHeight)
      }
    })), EditorView.updateListener.of(function (viewUpdate) {
      var _onUpdateRef$current;

      if (viewUpdate.docChanged) {
        var _onChangeRef$current;

        var _value = viewUpdate.state.doc.toString();

        (_onChangeRef$current = onChangeRef.current) === null || _onChangeRef$current === void 0 ? void 0 : _onChangeRef$current.call(onChangeRef, _value, viewUpdate);
      }

      (_onUpdateRef$current = onUpdateRef.current) === null || _onUpdateRef$current === void 0 ? void 0 : _onUpdateRef$current.call(onUpdateRef, viewUpdate);
    }) // todo placeholder 会导致起始位置无法输入中文
    ]);
  }, [height, minHeight, minHeight, props.extensions]); // reconfigure

  useEffect(function () {
    var _editorViewRef$curren;

    // todo 样式会一直累加
    (_editorViewRef$curren = editorViewRef.current) === null || _editorViewRef$curren === void 0 ? void 0 : _editorViewRef$curren.dispatch({
      effects: StateEffect.reconfigure.of(extensions)
    });
  }, [extensions]); // value change

  useEffect(function () {
    var _editorViewRef$curren2;

    var currentValue = (_editorViewRef$curren2 = editorViewRef.current) === null || _editorViewRef$curren2 === void 0 ? void 0 : _editorViewRef$curren2.state.doc.toString();

    if (value !== currentValue) {
      var _editorViewRef$curren3;

      (_editorViewRef$curren3 = editorViewRef.current) === null || _editorViewRef$curren3 === void 0 ? void 0 : _editorViewRef$curren3.dispatch({
        changes: {
          from: 0,
          to: currentValue === null || currentValue === void 0 ? void 0 : currentValue.length,
          insert: value
        }
      });
    }
  }, [value]); // 编辑器初始化

  useEffect(function () {
    // todo 第一次 setState containerRef.current 值会变
    if (containerRef.current) {
      var state = EditorState.create({
        doc: value,
        extensions: extensions
      });
      editorViewRef.current = new EditorView({
        state: state,
        parent: containerRef.current
      });
      return function () {
        var _editorViewRef$curren4;

        (_editorViewRef$curren4 = editorViewRef.current) === null || _editorViewRef$curren4 === void 0 ? void 0 : _editorViewRef$curren4.destroy();
      };
    }
  }, []);
  return editorViewRef;
});