function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

import { EditorView } from '@codemirror/basic-setup';
import debounce from 'lodash/debounce'; // @ts-ignore

import smoothscroll from 'smoothscroll-polyfill';
import { getCodeNode } from '../commands';
smoothscroll.polyfill();

function getNonBlankLine(view, line) {
  var state = view.state;
  var codeNode = getCodeNode(view, line.from);

  if ((codeNode === null || codeNode === void 0 ? void 0 : codeNode.name) === 'CodeBlock' || (codeNode === null || codeNode === void 0 ? void 0 : codeNode.name) === 'FencedCode') {
    return line;
  }

  var newLine = line;

  while (/^\s*$/.test(newLine.text)) {
    if (newLine.to >= state.doc.length) {
      break;
    }

    newLine = state.doc.lineAt(newLine.to + 1);
  }

  while (/^\s*$/.test(newLine.text)) {
    if (Math.min(line.from, newLine.from) <= 0) {
      break;
    }

    newLine = state.doc.lineAt(Math.min(line.from, newLine.from) - 1);
  }

  return newLine;
}

function getOffsetTop(el, parentEl) {
  var offsetTop = el.offsetTop;
  var pEL = el.offsetParent;
  var i = 0;

  while (pEL && pEL !== parentEl) {
    offsetTop += pEL.offsetTop;
    pEL = pEL.offsetParent;
    if (++i >= 100) break;
  }

  return offsetTop;
}

export default (function (editorRef) {
  var previewInterval = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1000;
  var lastCursorLineNumber = -1;
  var lockScrollEvent = false;
  var lastScrollLineNumber = -1;

  function scrollToLine(children, line) {
    var _text$match;

    var marginTop = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
    var el = arguments.length > 3 ? arguments[3] : undefined;
    var view = editorRef.current.getEditorView();
    var preview = editorRef.current.getPreview();
    var state = view.state;
    var previewDOM = preview.getContainer();
    if (!previewDOM) return;
    var scrollTop = 0;
    var offsetTop = 0;
    var index = -1;

    for (var i = 0; i < children.length; i++) {
      var _node$position, _node$position$start, _node$position2, _node$position2$end;

      var node = children[i];
      if (node.type === 'element') index++;
      var from = (_node$position = node.position) === null || _node$position === void 0 ? void 0 : (_node$position$start = _node$position.start) === null || _node$position$start === void 0 ? void 0 : _node$position$start.offset;
      var to = (_node$position2 = node.position) === null || _node$position2 === void 0 ? void 0 : (_node$position2$end = _node$position2.end) === null || _node$position2$end === void 0 ? void 0 : _node$position2$end.offset;

      if (isNaN(to)) {
        for (var j = i + 1; j < children.length; j++) {
          var _children$j$position, _children$j$position$;

          to = (_children$j$position = children[j].position) === null || _children$j$position === void 0 ? void 0 : (_children$j$position$ = _children$j$position.start) === null || _children$j$position$ === void 0 ? void 0 : _children$j$position$.offset;
        }
      }

      if (isNaN(from) || isNaN(to)) continue;

      if (from <= line.from && to >= line.to || line.from <= from && line.to >= to) {
        if (index > -1) {
          var _el;

          el = ((_el = el) === null || _el === void 0 ? void 0 : _el.children[index]) || previewDOM.children[index] || el;
        }

        if (!el) return;
        var elStyle = window.getComputedStyle(el);
        var lineHeight = parseInt(elStyle.lineHeight);

        switch (node.tagName) {
          case 'p':
            var text = state.sliceDoc(from, line.from);
            var lineNumber = ((_text$match = text.match(/\r?\n|<br\s*\/>/gi)) === null || _text$match === void 0 ? void 0 : _text$match.length) || 0;

            if (lineNumber > 0) {
              var _i = 0;
              var _lineHeight = lineHeight;

              var _iterator = _createForOfIteratorHelper(el.children),
                  _step;

              try {
                for (_iterator.s(); !(_step = _iterator.n()).done;) {
                  var _node = _step.value;

                  if (_node.tagName.toLowerCase() !== 'br') {
                    var nodeStyle = window.getComputedStyle(_node);

                    var _marginTop = parseFloat(nodeStyle.marginTop);

                    var marginBottom = parseFloat(nodeStyle.marginBottom);
                    _lineHeight = Math.max(_node.clientHeight + _marginTop + marginBottom, lineHeight);
                    continue;
                  } // offsetTop += _lineHeight;


                  offsetTop = _node.getBoundingClientRect().top - el.getBoundingClientRect().top + _lineHeight;
                  _lineHeight = lineHeight;

                  if (++_i >= lineNumber) {
                    break;
                  }
                } // el = [...el.children].filter(node => node.tagName.toLowerCase() === 'br')[lineNumber - 1] || el;

              } catch (err) {
                _iterator.e(err);
              } finally {
                _iterator.f();
              }
            } // el?.scrollIntoView({ behavior: 'smooth' });


            break;

          case 'pre':
            var codeNode = getCodeNode(view, line.from);

            if ((codeNode === null || codeNode === void 0 ? void 0 : codeNode.name) === 'CodeBlock' || (codeNode === null || codeNode === void 0 ? void 0 : codeNode.name) === 'FencedCode') {
              var _lineNumber = line.number - state.doc.lineAt(codeNode.from).number;

              var totalLineNumber = state.doc.lineAt(codeNode.to).number - state.doc.lineAt(codeNode.from).number + 1;

              if ((codeNode === null || codeNode === void 0 ? void 0 : codeNode.name) === 'FencedCode') {
                _lineNumber--;
                totalLineNumber -= 2;
              }

              if (_lineNumber > 0) {
                var codeEl = el.children[0];
                var paddingTop = 0;
                var paddingBottom = 0;

                if (codeEl) {
                  var codeElStyle = window.getComputedStyle(codeEl);
                  paddingTop = parseInt(codeElStyle.paddingTop);
                  paddingBottom = parseInt(codeElStyle.paddingBottom);
                  var codeContentHeight = codeEl.clientHeight - paddingTop - paddingBottom;
                  lineHeight = codeContentHeight / totalLineNumber;
                }

                offsetTop = paddingTop + _lineNumber * lineHeight;
              }
            }

            break;

          default:
            return scrollToLine(node.children || [], line, marginTop, el);
        }

        break;
      }
    }

    if (el) {
      scrollTop = getOffsetTop(el, previewDOM);
    }

    previewDOM.scrollTo({
      top: scrollTop + offsetTop + marginTop,
      behavior: 'smooth'
    }); // previewDOM.scrollTop = scrollTop + offsetTop + marginTop;

    lastScrollLineNumber = line.number;
  }

  function handleScrollToSelection() {
    var _editorRef$current, _editorRef$current2;

    var view = (_editorRef$current = editorRef.current) === null || _editorRef$current === void 0 ? void 0 : _editorRef$current.getEditorView();
    var preview = (_editorRef$current2 = editorRef.current) === null || _editorRef$current2 === void 0 ? void 0 : _editorRef$current2.getPreview();
    if (!view || !preview) return;
    var state = view.state;
    var value = state.doc.toString();
    if (value !== preview.getMarkdown()) return;
    var hast = preview.getHast();
    var line = getNonBlankLine(view, state.doc.lineAt(state.selection.main.head)); // todo 换行的可能不太精准

    var marginTop = /^\s*$/.test(state.sliceDoc(line.to)) ? 0 : view.scrollDOM.scrollTop - view.lineBlockAt(line.from).top;

    if (lastScrollLineNumber !== line.number) {
      scrollToLine(hast.children, line, marginTop);
    }
  }

  var handleDebounceScrollToSelection = debounce(handleScrollToSelection, previewInterval + 300);
  var handleDebounceScroll = debounce(function () {
    var _editorRef$current3, _editorRef$current4;

    var view = (_editorRef$current3 = editorRef.current) === null || _editorRef$current3 === void 0 ? void 0 : _editorRef$current3.getEditorView();
    var preview = (_editorRef$current4 = editorRef.current) === null || _editorRef$current4 === void 0 ? void 0 : _editorRef$current4.getPreview();
    if (!view || !preview) return;
    var state = view.state;
    var value = view.state.doc.toString();
    if (value !== preview.getMarkdown()) return;
    var hast = preview.getHast(); // 可见区域第一行

    var blockInfo = view.lineBlockAtHeight(view.scrollDOM.scrollTop);
    var visibleFirstLine = state.doc.lineAt(blockInfo.from);
    var line = getNonBlankLine(view, visibleFirstLine);

    if (lastScrollLineNumber !== line.number) {
      var marginTop = /^\s*$/.test(visibleFirstLine.text) ? 0 : view.scrollDOM.scrollTop - blockInfo.top;
      scrollToLine(hast.children, line, marginTop);
      lastScrollLineNumber = -1; // 让每次滚动事件都执行
    }
  }, 300);
  return [EditorView.updateListener.of(function (viewUpdate) {
    var _editorRef$current5;

    if (viewUpdate.selectionSet && ((_editorRef$current5 = editorRef.current) === null || _editorRef$current5 === void 0 ? void 0 : _editorRef$current5.getDisplayMode().editorPreview)) {
      var state = viewUpdate.state;
      var cursorLineNumber = state.doc.lineAt(state.selection.main.head).number;
      lockScrollEvent = viewUpdate.docChanged && cursorLineNumber !== lastCursorLineNumber;
      lastCursorLineNumber = cursorLineNumber;
      handleDebounceScroll.cancel();

      if (viewUpdate.docChanged) {
        handleDebounceScrollToSelection();
      } else {
        handleDebounceScrollToSelection.cancel();
        handleScrollToSelection();
      }
    }
  }), EditorView.domEventHandlers({
    scroll: function scroll() {
      var _editorRef$current6;

      if (!lockScrollEvent && ((_editorRef$current6 = editorRef.current) === null || _editorRef$current6 === void 0 ? void 0 : _editorRef$current6.getDisplayMode().editorPreview)) {
        handleDebounceScrollToSelection.cancel();
        handleDebounceScroll();
      }

      lockScrollEvent = false;
    }
  })];
});