import React from 'react';
import { EditorView } from '@codemirror/view';
import { UseCodeMirrorProps } from './hooks/useCodeMirror';
import { ToolbarInstance, ToolbarProps } from './Toolbar';
import { PreviewInstance } from './Preview';
import './Editor.less';
export interface EditorInstance {
  getContainer: () => HTMLDivElement | null;
  getEditorView: () => EditorView | null;
  getToolbar: () => ToolbarInstance | null;
  getPreview: () => PreviewInstance | null;
  getDisplayMode: () => {
    fullScreen: boolean;
    preview: boolean;
    editorPreview: boolean;
  };
  toggleFullScreen(bool?: boolean): void;
  togglePreview(bool?: boolean): void;
  toggleEditorPreview(bool?: boolean): void;
}
export interface EditorProps extends Omit<UseCodeMirrorProps, 'containerRef' | 'onUpdate'> {
  /**
   * @description  工具条
   */
  toolbar?:
    | false
    | {
        buttons: ToolbarProps['buttons'];
      };
  /**
   * @description 预览选项
   */
  previewOptions?: {
    /**
     * @description  刷新间隔(ms)
     * @default 1000
     */
    interval?: number;
    /**
     * @description 自定义渲染
     */
    render?(markdown: string): React.ReactNode;
    /**
     * @description remark 插件
     */
    remarkPlugins?: import('unified').PluggableList;
    /**
     * @description rehype 插件
     */
    rehypePlugins?: import('unified').PluggableList;
  };
}
declare const Editor: React.ForwardRefExoticComponent<
  EditorProps & React.RefAttributes<EditorInstance>
>;
export default Editor;
